// TODO: Use new class which takes arguments or options

(function() {
    'use strict';

    var yeoman = require('yeoman-generator');
    var chalk = require('chalk');
    var yosay = require('yosay');

    module.exports = yeoman.generators.Base.extend({

        initializing: function () {
            this.pkg = require('../../package.json');
        },

        prompting: function () {
            var done = this.async();

            // Have Yeoman greet the user.
            this.log(yosay('Welcome to the classy ' + chalk.red('AmdNpmJsModule') + ' generator!'));

            var prompts = [
                {
                    type: 'input',
                    name: 'packageName',
                    message: 'What would you like to call the package?',
                    default: this.appname
                },
                {
                    type: 'input',
                    name: 'className',
                    message: 'What would you like to name the class?',
                    default: this.appname
                }
                // TODO: author name, license, bug url, homepage
            ];

            this.prompt(prompts, function (props) {
                this.className = props.className;
                this.packageName = props.packageName;

                this._resolvers = {
                    packageName: this.packageName,
                    className: this.className
                };

                done();
            }.bind(this));
        },


        writing: {
            app: function () {

                this.fs.copyTpl(
                    this.templatePath('_package.json'),
                    this.destinationPath('package.json'),
                    this._resolvers
                );
                this.fs.copyTpl(
                    this.templatePath('_bower.json'),
                    this.destinationPath('bower.json'),
                    this._resolvers
                );

                this.fs.copyTpl(
                    this.templatePath('_.travis.yml'),
                    this.destinationPath('.travis.yml'),
                    this._resolvers
                );
                this.fs.copyTpl(
                    this.templatePath('_.yo-rc.json'),
                    this.destinationPath('.yo-rc.json'),
                    this._resolvers
                );

                // Copy the replacements
                this.fs.copy(
                    this.templatePath('replacements.json'),
                    this.destinationPath('replacements.json'),
                    this._resolvers
                );

            },

            subgenerators: function() {
                this.composeWith('amd-npm-js-module:module', {
                    options: {
                        className: this.className,
                        path: 'src/'
                    }
                });

                this.composeWith('amd-npm-js-module:gruntfile', {
                    options: {className: this.className}
                });

                this.composeWith('amd-npm-js-module:test', {
                    options: {className: this.className}
                });
            },

            projectfiles: function () {
                this.fs.copy(
                    this.templatePath('_jshintrc'),
                    this.destinationPath('.jshintrc')
                );
            }
        },

        install: function () {
            this.installDependencies({
                skipInstall: this.options['skip-install']
            });
        }
    });
})();